// based on the PCRE C++ wrapper classes : http://www.daemon.de/PCRE

// important note* , the global flag for match behaves differntly than the 
// traditional global, given a buffer with multiple regex matches ( AND substrings! )
// they will all be collected and stored in matches


// Charles Sanders, 8.31.2004


module easypcre;


private {
	import std.stream;
	import std.string;
	import pcre;

	alias char [] string;
	alias std.string.toString  d_str;
	alias std.string.toStringz c_str;
}

class PCREException 
{
	int error;
	string strError;

	this ( string s ) { strError = s; }
	this ( int x = 0 ) { error = x; } 
	string toString ()
	{

		if ( strError.length ) 		return strError;
		else {
			string msg;

			switch(error) {
			case 0: msg = "General PCRE Error";      break;
			case -1: msg = "PCRE_ERROR_NOMATCH";      break;
			case -2: msg = "PCRE_ERROR_null";         break;
			case -3: msg = "PCRE_ERROR_BADOPTION";    break;
			case -4: msg = "PCRE_ERROR_BADMAGIC";     break;
			case -5: msg = "PCRE_ERROR_UNKNOWN_NODE"; break;
			case -6: msg = "PCRE_ERROR_NOMEMORY";     break;
			case -7: msg = "PCRE_ERROR_NOSUBSTRING";  break;
				// pcre4-HINT: add PCRE_ERROR_MATCHLIMIT support
			}
			return msg;
		}
	}

}

class EasyRegex
{
public:
	string expression;

	bit globalOption = false;
	uint regexFlags = 0;

	real_pcre* regex = null;              
	pcre_extra *regex_extra = null;

	int sub_len;
	int* sub_vec = null;

	int erroffset;
	char* err_str = null;
	
	string [] matches;

public:
	this (string exp, string flags = "" ) {
		
		expression   = exp;
		if(expression[0] != '(' && expression[ expression.length - 1 ] != ')' )  
			expression = "(" ~ expression ~ ")";


		for(int flag=0; flag<flags.length; flag++) {
			switch(flags[flag]) {
			case 'i': regexFlags  |= PCRE_CASELESS; break;
			case 'm': regexFlags |= PCRE_MULTILINE;                  break;
			case 's': regexFlags |= PCRE_DOTALL;                     break;
			case 'x': regexFlags |= PCRE_EXTENDED;                   break;
			case 'g': globalOption = true; break;
			}
		}


		zero();
		compile(regexFlags);

	}

	bit match(string stuff ) {
		if ( !globalOption ) return singleMatch(stuff );
		else return globalMatch(stuff );
	}

	string [] split(in string buffer, ulong limit = ulong.max ) {

		string [] ret;
		int startPos = 0, endPos = 0;
		int numMatches = 0;

		while ( true ) 
		{
			if ( singleMatch(buffer[startPos .. buffer.length]  ) == true )
			{
				if ( matches.length > 0 ) 
				{

					endPos = getMatchStart() + startPos;

				
					debug {

// 						stdout.writeLine("Start Pos : " ~ d_str(startPos ) );
// 						stdout.writeLine("End Pos : " ~ d_str(endPos ) );
// 						stdout.writeLine("Match : " ~ buffer[startPos .. endPos] ~ "\n\n" );

					}

					if ( numMatches++ > limit  ) break; // break if weve reached the limit
					
					ret ~= buffer[startPos .. endPos];
					startPos += getMatchStart() + matches[0].length;
					
				}
				else break;
			}
		
			else {

				ret ~= buffer[startPos ..buffer.length];
				break;
			}
		}

		return ret;
	}


	string replace( in string buffer, in string replaceText) {
		
		string returnText = buffer;
		

		int startPos = 0, endPos = 0;
		int numMatches = 0;
		bit found = false;
		while ( singleMatch(buffer[startPos .. buffer.length] ) )

		{
			if ( matches.length > 0 ) 
			{
				startPos += getMatchStart() + matches[0].length;
				returnText = std.string.replace(returnText,matches[0],replaceVars(replaceText)) ;
			}
			else break;
		}

		return returnText;
	}


	string replaceVars(in string replaceText) {
		string text = replaceText;
		for ( int i = 0;i < matches.length;i++ ) 
		{
			string dollar = "$" ~ d_str(i);
			text = std.string.replace(text,dollar,matches[i] );

		}
		return text;
	}

	void reset(char [] exp, char [] flags = "" )
	{

		
		if ( regex ) delete regex;
		regex = null;        
		if ( regex_extra) delete regex_extra;
		regex_extra = null;  
		if ( err_str ) delete err_str;
		err_str = null;


		
		expression   = exp;
		if(expression[0] != '(' && expression[ expression.length - 1 ] != ')' )  
			expression = "(" ~ expression ~ ")";


		for(int flag=0; flag<flags.length; flag++) {
			switch(flags[flag]) {
			case 'i': regexFlags  |= PCRE_CASELESS; break;
			case 'm': regexFlags |= PCRE_MULTILINE;                  break;
			case 's': regexFlags |= PCRE_DOTALL;                     break;
			case 'x': regexFlags |= PCRE_EXTENDED;                   break;
			case 'g': globalOption = true; break;
			}
		}


		zero();
		compile(regexFlags);


	}

protected:


	bit singleMatch(in string stuff) {

		zero();
		if (sub_vec != null)
			delete sub_vec;

		sub_vec = new int[sub_len];

		int num = pcre_exec(regex, regex_extra, c_str(stuff),
				    cast(int)stuff.length, 0, 0, cast(int *)sub_vec, sub_len);


		if(num <= 0) return false;

		else if(num == 1) {

			return true;

		}
		else if(num > 1) {

			char **stringlist;

			int res = pcre_get_substring_list(c_str(stuff), sub_vec, num, &stringlist);
			if(res == 0) {
				for(int i=1; i<num; i++) {
					matches ~= d_str(stringlist[i]).dup;
				}
				pcre_free_substring_list(stringlist);
			}
			else {
				throw new PCREException(res);
			}
			return true;
		}
		else {
			return false;
		}
	}

	bit globalMatch( string buffer )
	{

		string [] ret;
		int startPos = 0, endPos = 0;
		int numMatches = 0;
		bit found = false;
		while ( singleMatch(buffer[startPos .. buffer.length] ) )

		{

			found = true;
			if ( matches.length > 0 ) 
			{
				ret ~= matches;
				startPos += getMatchStart() + matches[0].length;
				zero();
			}
			else break;
		}
		matches = ret;



		return found;

		
	}


	
	int getMatchStart(int pos = -1) {
		if ( pos == -1 ) 
			return sub_vec ? sub_vec[0] : -1;
		else {
			if(pos >= 0 && pos <= matches.length) {
				return sub_vec[ (++pos) * 2 ];
			}
			else {
				throw new PCREException("out of range");
			}  
		}
	}


	int getMatchEnd(int pos = -1) {
		if ( pos == -1 ) 
			return sub_vec ? sub_vec[1] - 1 : -1;
		else {
			if(pos >= 0 && pos <= matches.length) {
				return sub_vec[ ((++pos) * 2) + 1 ] - 1;
			}
			else {
				throw new PCREException("out of range");
			}
		}

	}



	void compile (int flags )
	{

		regex       = pcre_compile(c_str(expression), flags,
					    cast(char **)(&err_str), &erroffset, null);

		if(regex == null) throw new PCREException(d_str(err_str) );
		

		int where = 0 ;
		int info = pcre_fullinfo( regex, regex_extra, PCRE_INFO_CAPTURECOUNT, &where);

		if(info == 0) {
			sub_len = (where +2) * 3; /* see "man pcre" for the exact formula */
		}
		else {
			throw new PCREException(info);
		}
		zero();
	}

	void zero () 
	{
		// whats the best way to do this ??
		matches.length = 0;
	}


}


unittest {


	EasyRegex reg = new EasyRegex(CommonRegex.validIPV4Address);
	string buffer = "127.0.0.1";
	assert(reg.match(buffer) );
	assert(reg.matches[1] == "127");
	assert(reg.matches[4] == "1");
	reg.reset(`((\d{3})-(\d{3})-(\d{4}))`);
	buffer = "512-555-5555";
	assert(reg.match(buffer));
	assert(reg.matches[1] == "512");
	assert(reg.matches[2] == "555");

}


// see also 

// Regex::common
// http://aspn.activestate.com/ASPN/Cookbook/Rx/

struct CommonRegex
{


//need to fill this up, user contributions welcome !

	static string urlLinks = `<\s*a\s*href=["'](.*?)["'].*?>`; // to grab valid html links , $1 is actual link
	static string htmlContents = `<([^>])+>|&([^;])+;`; // to extract all <tag> plain text </tag> from html
	static string validIPV4Address = `(^([01]?\d\d?|2[0-4]\d|25[0-5])\.([01]?\d\d?|2[0-4]\d|25[0-5])\.([01]?\d\d?|2[0-4]\d|25[0-5])\.([01]?\d\d?|2[0-4]\d|25[0-5]))`; // for valid ip address, $1 - $4 are the dotted quads
				    // yes i know its ridicoulous
        static string validEmailPattern = `[\040\t]*(?:\([^\\\x80-\xff\n\015()]*(?:(?:\\[^\x80-\xff]|\([^\\\x80-\xff\n\015()]*(?:\\[^\x80-\xff][^\\\x80-\xff\n\015()]*)*\))[^\\\x80-\xff\n\015()]*)*\)[\040\t]*)*(?:(?:[^(\040)<>@,;:".\\\[\]\000-\037\x80-\xff]+(?![^(\040)<>@,;:".\\\[\]\000-\037\x80-\xff])|"[^\\\x80-\xff\n\015"]*(?:\\[^\x80-\xff][^\\\x80-\xff\n\015"]*)*")[\040\t]*(?:\([^\\\x80-\xff\n\015()]*(?:(?:\\[^\x80-\xff]|\([^\\\x80-\xff\n\015()]*(?:\\[^\x80-\xff][^\\\x80-\xff\n\015()]*)*\))[^\\\x80-\xff\n\015()]*)*\)[\040\t]*)*(?:\.[\040\t]*(?:\([^\\\x80-\xff\n\015()]*(?:(?:\\[^\x80-\xff]|\([^\\\x80-\xff\n\015()]*(?:\\[^\x80-\xff][^\\\x80-\xff\n\015()]*)*\))[^\\\x80-\xff\n\015()]*)*\)[\040\t]*)*(?:[^(\040)<>@,;:".\\\[\]\000-\037\x80-\xff]+(?![^(\040)<>@,;:".\\\[\]\000-\037\x80-\xff])|"[^\\\x80-\xff\n\015"]*(?:\\[^\x80-\xff][^\\\x80-\xff\n\015"]*)*")[\040\t]*(?:\([^\\\x80-\xff\n\015()]*(?:(?:\\[^\x80-\xff]|\([^\\\x80-\xff\n\015()]*(?:\\[^\x80-\xff][^\\\x80-\xff\n\015()]*)*\))[^\\\x80-\xff\n\015()]*)*\)[\040\t]*)*)*@[\040\t]*(?:\([^\\\x80-\xff\n\015()]*(?:(?:\\[^\x80-\xff]|\([^\\\x80-\xff\n\015()]*(?:\\[^\x80-\xff][^\\\x80-\xff\n\015()]*)*\))[^\\\x80-\xff\n\015()]*)*\)[\040\t]*)*(?:[^(\040)<>@,;:".\\\[\]\000-\037\x80-\xff]+(?![^(\040)<>@,;:".\\\[\]\000-\037\x80-\xff])|\[(?:[^\\\x80-\xff\n\015\[\]]|\\[^\x80-\xff])*\])[\040\t]*(?:\([^\\\x80-\xff\n\015()]*(?:(?:\\[^\x80-\xff]|\([^\\\x80-\xff\n\015()]*(?:\\[^\x80-\xff][^\\\x80-\xff\n\015()]*)*\))[^\\\x80-\xff\n\015()]*)*\)[\040\t]*)*(?:\.[\040\t]*(?:\([^\\\x80-\xff\n\015()]*(?:(?:\\[^\x80-\xff]|\([^\\\x80-\xff\n\015()]*(?:\\[^\x80-\xff][^\\\x80-\xff\n\015()]*)*\))[^\\\x80-\xff\n\015()]*)*\)[\040\t]*)*(?:[^(\040)<>@,;:".\\\[\]\000-\037\x80-\xff]+(?![^(\040)<>@,;:".\\\[\]\000-\037\x80-\xff])|\[(?:[^\\\x80-\xff\n\015\[\]]|\\[^\x80-\xff])*\])[\040\t]*(?:\([^\\\x80-\xff\n\015()]*(?:(?:\\[^\x80-\xff]|\([^\\\x80-\xff\n\015()]*(?:\\[^\x80-\xff][^\\\x80-\xff\n\015()]*)*\))[^\\\x80-\xff\n\015()]*)*\)[\040\t]*)*)*|(?:[^(\040)<>@,;:".\\\[\]\000-\037\x80-\xff]+(?![^(\040)<>@,;:".\\\[\]\000-\037\x80-\xff])|"[^\\\x80-\xff\n\015"]*(?:\\[^\x80-\xff][^\\\x80-\xff\n\015"]*)*")[^()<>@,;:".\\\[\]\x80-\xff\000-\010\012-\037]*(?:(?:\([^\\\x80-\xff\n\015()]*(?:(?:\\[^\x80-\xff]|\([^\\\x80-\xff\n\015()]*(?:\\[^\x80-\xff][^\\\x80-\xff\n\015()]*)*\))[^\\\x80-\xff\n\015()]*)*\)|"[^\\\x80-\xff\n\015"]*(?:\\[^\x80-\xff][^\\\x80-\xff\n\015"]*)*")[^()<>@,;:".\\\[\]\x80-\xff\000-\010\012-\037]*)*<[\040\t]*(?:\([^\\\x80-\xff\n\015()]*(?:(?:\\[^\x80-\xff]|\([^\\\x80-\xff\n\015()]*(?:\\[^\x80-\xff][^\\\x80-\xff\n\015()]*)*\))[^\\\x80-\xff\n\015()]*)*\)[\040\t]*)*(?:@[\040\t]*(?:\([^\\\x80-\xff\n\015()]*(?:(?:\\[^\x80-\xff]|\([^\\\x80-\xff\n\015()]*(?:\\[^\x80-\xff][^\\\x80-\xff\n\015()]*)*\))[^\\\x80-\xff\n\015()]*)*\)[\040\t]*)*(?:[^(\040)<>@,;:".\\\[\]\000-\037\x80-\xff]+(?![^(\040)<>@,;:".\\\[\]\000-\037\x80-\xff])|\[(?:[^\\\x80-\xff\n\015\[\]]|\\[^\x80-\xff])*\])[\040\t]*(?:\([^\\\x80-\xff\n\015()]*(?:(?:\\[^\x80-\xff]|\([^\\\x80-\xff\n\015()]*(?:\\[^\x80-\xff][^\\\x80-\xff\n\015()]*)*\))[^\\\x80-\xff\n\015()]*)*\)[\040\t]*)*(?:\.[\040\t]*(?:\([^\\\x80-\xff\n\015()]*(?:(?:\\[^\x80-\xff]|\([^\\\x80-\xff\n\015()]*(?:\\[^\x80-\xff][^\\\x80-\xff\n\015()]*)*\))[^\\\x80-\xff\n\015()]*)*\)[\040\t]*)*(?:[^(\040)<>@,;:".\\\[\]\000-\037\x80-\xff]+(?![^(\040)<>@,;:".\\\[\]\000-\037\x80-\xff])|\[(?:[^\\\x80-\xff\n\015\[\]]|\\[^\x80-\xff])*\])[\040\t]*(?:\([^\\\x80-\xff\n\015()]*(?:(?:\\[^\x80-\xff]|\([^\\\x80-\xff\n\015()]*(?:\\[^\x80-\xff][^\\\x80-\xff\n\015()]*)*\))[^\\\x80-\xff\n\015()]*)*\)[\040\t]*)*)*(?:,[\040\t]*(?:\([^\\\x80-\xff\n\015()]*(?:(?:\\[^\x80-\xff]|\([^\\\x80-\xff\n\015()]*(?:\\[^\x80-\xff][^\\\x80-\xff\n\015()]*)*\))[^\\\x80-\xff\n\015()]*)*\)[\040\t]*)*@[\040\t]*(?:\([^\\\x80-\xff\n\015()]*(?:(?:\\[^\x80-\xff]|\([^\\\x80-\xff\n\015()]*(?:\\[^\x80-\xff][^\\\x80-\xff\n\015()]*)*\))[^\\\x80-\xff\n\015()]*)*\)[\040\t]*)*(?:[^(\040)<>@,;:".\\\[\]\000-\037\x80-\xff]+(?![^(\040)<>@,;:".\\\[\]\000-\037\x80-\xff])|\[(?:[^\\\x80-\xff\n\015\[\]]|\\[^\x80-\xff])*\])[\040\t]*(?:\([^\\\x80-\xff\n\015()]*(?:(?:\\[^\x80-\xff]|\([^\\\x80-\xff\n\015()]*(?:\\[^\x80-\xff][^\\\x80-\xff\n\015()]*)*\))[^\\\x80-\xff\n\015()]*)*\)[\040\t]*)*(?:\.[\040\t]*(?:\([^\\\x80-\xff\n\015()]*(?:(?:\\[^\x80-\xff]|\([^\\\x80-\xff\n\015()]*(?:\\[^\x80-\xff][^\\\x80-\xff\n\015()]*)*\))[^\\\x80-\xff\n\015()]*)*\)[\040\t]*)*(?:[^(\040)<>@,;:".\\\[\]\000-\037\x80-\xff]+(?![^(\040)<>@,;:".\\\[\]\000-\037\x80-\xff])|\[(?:[^\\\x80-\xff\n\015\[\]]|\\[^\x80-\xff])*\])[\040\t]*(?:\([^\\\x80-\xff\n\015()]*(?:(?:\\[^\x80-\xff]|\([^\\\x80-\xff\n\015()]*(?:\\[^\x80-\xff][^\\\x80-\xff\n\015()]*)*\))[^\\\x80-\xff\n\015()]*)*\)[\040\t]*)*)*)*:[\040\t]*(?:\([^\\\x80-\xff\n\015()]*(?:(?:\\[^\x80-\xff]|\([^\\\x80-\xff\n\015()]*(?:\\[^\x80-\xff][^\\\x80-\xff\n\015()]*)*\))[^\\\x80-\xff\n\015()]*)*\)[\040\t]*)*)?(?:[^(\040)<>@,;:".\\\[\]\000-\037\x80-\xff]+(?![^(\040)<>@,;:".\\\[\]\000-\037\x80-\xff])|"[^\\\x80-\xff\n\015"]*(?:\\[^\x80-\xff][^\\\x80-\xff\n\015"]*)*")[\040\t]*(?:\([^\\\x80-\xff\n\015()]*(?:(?:\\[^\x80-\xff]|\([^\\\x80-\xff\n\015()]*(?:\\[^\x80-\xff][^\\\x80-\xff\n\015()]*)*\))[^\\\x80-\xff\n\015()]*)*\)[\040\t]*)*(?:\.[\040\t]*(?:\([^\\\x80-\xff\n\015()]*(?:(?:\\[^\x80-\xff]|\([^\\\x80-\xff\n\015()]*(?:\\[^\x80-\xff][^\\\x80-\xff\n\015()]*)*\))[^\\\x80-\xff\n\015()]*)*\)[\040\t]*)*(?:[^(\040)<>@,;:".\\\[\]\000-\037\x80-\xff]+(?![^(\040)<>@,;:".\\\[\]\000-\037\x80-\xff])|"[^\\\x80-\xff\n\015"]*(?:\\[^\x80-\xff][^\\\x80-\xff\n\015"]*)*")[\040\t]*(?:\([^\\\x80-\xff\n\015()]*(?:(?:\\[^\x80-\xff]|\([^\\\x80-\xff\n\015()]*(?:\\[^\x80-\xff][^\\\x80-\xff\n\015()]*)*\))[^\\\x80-\xff\n\015()]*)*\)[\040\t]*)*)*@[\040\t]*(?:\([^\\\x80-\xff\n\015()]*(?:(?:\\[^\x80-\xff]|\([^\\\x80-\xff\n\015()]*(?:\\[^\x80-\xff][^\\\x80-\xff\n\015()]*)*\))[^\\\x80-\xff\n\015()]*)*\)[\040\t]*)*(?:[^(\040)<>@,;:".\\\[\]\000-\037\x80-\xff]+(?![^(\040)<>@,;:".\\\[\]\000-\037\x80-\xff])|\[(?:[^\\\x80-\xff\n\015\[\]]|\\[^\x80-\xff])*\])[\040\t]*(?:\([^\\\x80-\xff\n\015()]*(?:(?:\\[^\x80-\xff]|\([^\\\x80-\xff\n\015()]*(?:\\[^\x80-\xff][^\\\x80-\xff\n\015()]*)*\))[^\\\x80-\xff\n\015()]*)*\)[\040\t]*)*(?:\.[\040\t]*(?:\([^\\\x80-\xff\n\015()]*(?:(?:\\[^\x80-\xff]|\([^\\\x80-\xff\n\015()]*(?:\\[^\x80-\xff][^\\\x80-\xff\n\015()]*)*\))[^\\\x80-\xff\n\015()]*)*\)[\040\t]*)*(?:[^(\040)<>@,;:".\\\[\]\000-\037\x80-\xff]+(?![^(\040)<>@,;:".\\\[\]\000-\037\x80-\xff])|\[(?:[^\\\x80-\xff\n\015\[\]]|\\[^\x80-\xff])*\])[\040\t]*(?:\([^\\\x80-\xff\n\015()]*(?:(?:\\[^\x80-\xff]|\([^\\\x80-\xff\n\015()]*(?:\\[^\x80-\xff][^\\\x80-\xff\n\015()]*)*\))[^\\\x80-\xff\n\015()]*)*\)[\040\t]*)*)*>)`;

}



