extern (C) {
	const uint PCRE_MAJOR =          4;
	const uint PCRE_MINOR =           1;

	const uint PCRE_CASELESS           = 0x0001;
	const uint PCRE_MULTILINE          = 0x0002;
	const uint PCRE_DOTALL             = 0x0004;
	const uint PCRE_EXTENDED           = 0x0008;
	const uint PCRE_ANCHORED           = 0x0010;
	const uint PCRE_DOLLAR_ENDONLY     = 0x0020;
	const uint PCRE_EXTRA              = 0x0040;
	const uint PCRE_NOTBOL             = 0x0080;
	const uint PCRE_NOTEOL             = 0x0100;
	const uint PCRE_UNGREEDY           = 0x0200;
	const uint PCRE_NOTEMPTY           = 0x0400;
	const uint PCRE_UTF8               = 0x0800;
	const uint PCRE_NO_AUTO_CAPTURE    = 0x1000;

/* Exec-time and get/set-time error codes */

	const int PCRE_ERROR_NOMATCH        = (- 1);
	const int PCRE_ERROR_NULL           = (- 2);
	const int PCRE_ERROR_BADOPTION      = (- 3);
	const int PCRE_ERROR_BADMAGIC       = (- 4);
	const int PCRE_ERROR_UNKNOWN_NODE   = (- 5);
	const int PCRE_ERROR_NOMEMORY       = (- 6);
	const int PCRE_ERROR_NOSUBSTRING    = (- 7);
	const int PCRE_ERROR_MATCHLIMIT     = (- 8);
	const int PCRE_ERROR_CALLOUT        = (-9)  /* Never used by PCRE = itself */;

/* Request types for pcre_fullinfo() */

	const uint PCRE_INFO_OPTIONS            = 0;
	const uint PCRE_INFO_SIZE               = 1;
	const uint PCRE_INFO_CAPTURECOUNT       = 2;
	const uint PCRE_INFO_BACKREFMAX         = 3;
	const uint PCRE_INFO_FIRSTBYTE          = 4;
	const uint PCRE_INFO_FIRSTCHAR          = 4  /* For backwards = compatibility */;
	const uint PCRE_INFO_FIRSTTABLE         = 5;
	const uint PCRE_INFO_LASTLITERAL        = 6;
	const uint PCRE_INFO_NAMEENTRYSIZE      = 7;
	const uint PCRE_INFO_NAMECOUNT          = 8;
	const uint PCRE_INFO_NAMETABLE          = 9;
	const uint PCRE_INFO_STUDYSIZE         = 10;

/* Request types for pcre_config() */

	const uint PCRE_CONFIG_UTF8                    = 0;
	const uint PCRE_CONFIG_NEWLINE                 = 1;
	const uint PCRE_CONFIG_LINK_SIZE               = 2;
	const uint PCRE_CONFIG_POSIX_MALLOC_THRESHOLD  = 3;
	const uint POSIX_MALLOC_THRESHOLD  = 3;
	const uint PCRE_CONFIG_MATCH_LIMIT             = 4;

/* Bit flags for the pcre_extra structure */

	const uint PCRE_EXTRA_STUDY_DATA          = 0x0001;
	const uint PCRE_EXTRA_MATCH_LIMIT         = 0x0002;
	const uint PCRE_EXTRA_CALLOUT_DATA        = 0x0004;

	struct real_pcre {
		ulong magic_number;
		size_t size;                        /* Total that was malloced */
		ubyte *tables;        /* Pointer to tables */
		ulong options;
		uint top_bracket;
		uint top_backref;
		uint first_byte;
		uint req_byte;
		uint name_entry_size; /* Size of any name items; 0 => none */
		uint name_count;      /* Number of name items */
	};

	alias real_pcre pcre;

	struct pcre_extra {
		ulong flags;        /* Bits for which fields are set */
		void *study_data;               /* Opaque data from pcre_study() */
		ulong match_limit;  /* Maximum number of calls to match() */
		void *callout_data;             /* Data passed back in callouts */
	};

/* The structure for passing out data via the pcre_callout_function. We use a
   structure so that new fields can be added on the end in future versions,
   without changing the API of the function, thereby allowing old clients to work
   without modification. */

	struct pcre_callout_block {
		int          _version;           /* Identifies version of block */
		/* ------------------------ Version 0 ------------------------------- */
		int          callout_number;    /* Number compiled into pattern */
		int         *offset_vector;     /* The offset vector */
		 char  *subject;           /* The subject being matched */
		int          subject_length;    /* The length of the subject */
		int          start_match;       /* Offset to start of this match attempt */
		int          current_position;  /* Where we currently are */
		int          capture_top;       /* Max current capture */
		int          capture_last;      /* Most recently closed capture */
		void        *callout_data;      /* Data passed in with the call */
		/* ------------------------------------------------------------------ */
	};

/* Indirection for store get and free functions. These can be set to
   alternative malloc/free functions if required. There is also an optional
   callout function that is triggered by the (?) regex item. Some magic is
   required for Win32 DLL; it is null on other OS. For Virtual Pascal, these
   have to be different again. */


	void *pcre_malloc(size_t);
	void  pcre_free(void *);
	int   pcre_callout(pcre_callout_block *);

/* Exported PCRE functions */

	pcre *pcre_compile( char *, int,  char **,
			    int *,  ubyte *);
	int  pcre_config(int, void *);
	int  pcre_copy_named_substring( pcre *,  char *,
				       int *, int,  char *, char *, int);
	int  pcre_copy_substring( char *, int *, int, int,
				 char *, int);
	int  pcre_exec( pcre *,  pcre_extra *,
		        char *, int, int, int, int *, int);
	void pcre_free_substring( char *);
	void pcre_free_substring_list( char **);
	int  pcre_fullinfo( pcre *,  pcre_extra *, int,
			   void *);
	int  pcre_get_named_substring( pcre *,  char *,
				      int *, int,   char *,  char **);
	int  pcre_get_stringnumber( pcre *,  char *);
	int  pcre_get_substring( char *, int *, int, int,
				 char **);
	int  pcre_get_substring_list( char *, int *, int,
				      char ***);
	int  pcre_info( pcre *, int *, int *);
	ubyte *pcre_maketables();
	pcre_extra *pcre_study( pcre *, int,  char **);
	char *pcre_version();

}
